@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -34,7 +35,7 @@ namespace RC {
 
 const Arg_parser * parserp;
 
-// priority of _default_buffer_options by place of specification:
+// priority of default_buffer_options_ by place of specification:
 // ( 0 = lowest, 3 = highest )
 // 0 default priority
 // 1 priority of option specified in rc file as 'default file option'
@@ -61,8 +62,8 @@ struct Regex_options
   };
 
 
-Options _editor_options;
-Buffer::Options _default_buffer_options;
+Options editor_options_;
+Buffer::Options default_buffer_options_;
 Buffer_options_priorities priorities;
 std::vector< Regex_options > regex_options_vector;
 
@@ -173,11 +174,11 @@ bool set_int_option( int & option, const std::string & arg,
 
 bool set_option_from_code( const int code, const std::string & arg,
                            Options *edop, Buffer::Options *bufop,
-                           int priority = 0, const char * msg = "" )
+                           int priority = 0, const char * const msg = "" )
   {
   if( ( is_editor_option( code ) && !edop ) ||
       ( is_buffer_option( code ) && !bufop ) )
-    { std::fprintf( stderr, "%serror: option `%s' not allowed here\n", msg, optname( code ) );
+    { std::fprintf( stderr, "%serror: Option `%s' not allowed here.\n", msg, optname( code ) );
       return false; }
 
   int errcode = 0;
@@ -240,20 +241,20 @@ bool set_option_from_code( const int code, const std::string & arg,
 
   if( !errcode ) return true;
   if( errcode == 1 )
-    std::fprintf( stderr, "%serror: bad argument for option `%s'\n", msg, optname( code ) );
+    std::fprintf( stderr, "%serror: Bad argument for option `%s'.\n", msg, optname( code ) );
   else if( errcode == 2 )
-    std::fprintf( stderr, "%serror: value out of range for option `%s'\n", msg, optname( code ) );
+    std::fprintf( stderr, "%serror: Value out of range for option `%s'.\n", msg, optname( code ) );
   return false;
   }
 
 } // end namespace RC
 
 
-RC::Options & RC::editor_options() throw() { return _editor_options; }
+RC::Options & RC::editor_options() throw() { return editor_options_; }
 
 
 const Buffer::Options & RC::default_buffer_options() throw()
-  { return _default_buffer_options; }
+  { return default_buffer_options_; }
 
 
 void RC::apply_regex_options( Buffer & buffer ) throw()
@@ -319,14 +320,14 @@ int RC::process_rcfile( const std::string & name )
         const int ch = std::fgetc( f );
         if( ch == EOF || ch == '\n' ) break;
         }
-      std::fprintf( stderr, "\n%s %d: error: line too long\n",
+      std::fprintf( stderr, "\n%s %d: error: Line too long\n",
                     name.c_str(), line );
       retval = 2;
       }
     if( len == 0 || std::isspace( buf[0] ) ) continue;
     if( buf[0] != '-' )		// Create new file name dependent options
       {
-      const char * regex = std::strtok( buf, delimiters );
+      const char * const regex = std::strtok( buf, delimiters );
       if( regex_status != 1 )
         regex_options_vector.push_back( Regex_options() );
       regex_options_vector.back().regex_vector.push_back( regex );
@@ -334,17 +335,17 @@ int RC::process_rcfile( const std::string & name )
       }
     else			// Set an option
       {
-      const char * opt = std::strtok( buf, delimiters );
-      const char * arg = std::strtok( 0, delimiters );
+      const char * const opt = std::strtok( buf, delimiters );
+      const char * const arg = std::strtok( 0, delimiters );
       char msg[80];
       snprintf( msg, sizeof msg, "\n%s %d: ", name.c_str(), line );
-      Arg_parser parser( opt, arg, options );
+      const Arg_parser parser( opt, arg, options );
       if( parser.error().size() || !parser.arguments() || !parser.code( 0 ) )
         { std::fprintf( stderr, "%s%s\n", msg, parser.error().c_str() );
           retval = 2; }
       else if( regex_status == 0 )
         { if( !set_option_from_code( parser.code( 0 ), parser.argument( 0 ),
-                                     &_editor_options, &_default_buffer_options,
+                                     &editor_options_, &default_buffer_options_,
                                      1, msg ) ) retval = 2; }
       else
         {
@@ -377,10 +378,10 @@ int RC::process_options()
       {
       if( arg[0] != '+' ) break;			// filename
       if( parse_int( arg, line ) && --line >= 0 ) continue;
-      show_error( "bad line number" ); return 1;
+      show_error( "Bad line number." ); return 1;
       }
-    if( !set_option_from_code( code, arg, &_editor_options,
-                               &_default_buffer_options, 3,
+    if( !set_option_from_code( code, arg, &editor_options_,
+                               &default_buffer_options_, 3,
                                first_post ? "\n" : "" ) )
       { first_post = false; return 1; }
     }
@@ -395,13 +396,13 @@ int RC::process_options()
     if( Menu::is_regular( name, true ) )
       {
       try {
-        const int i = Bufhandle_vector::find_or_add_handle( _default_buffer_options, &name, -1, line );
+        const int i = Bufhandle_vector::find_or_add_handle( default_buffer_options_, &name, -1, line );
         bufferp = &Bufhandle_vector::handle( i ).buffer();
         }
       catch( Buffer::Error e )
         {
         if( first_post ) { first_post = false; std::fprintf( stderr, "\n" ); }
-        std::fprintf( stderr, "warning: `%s': %s\n", name.c_str(), e.s );
+        std::fprintf( stderr, "warning: `%s': %s\n", name.c_str(), e.msg );
         read_error_found = true;
         }
       }
@@ -421,7 +422,7 @@ int RC::process_options()
         if( arg[0] != '+' ) break;			// filename
         if( parse_int( arg, line ) && --line >= 0 ) continue;
         if( first_post ) { first_post = false; std::fprintf( stderr, "\n" ); }
-        show_error( "bad line number" ); return 1;
+        show_error( "Bad line number." ); return 1;
         }
       if( bufferp &&
           !set_option_from_code( code, parserp->argument( argind ),
@@ -435,7 +436,7 @@ int RC::process_options()
   catch( Buffer::Error e )
     {
     if( first_post ) { first_post = false; std::fprintf( stderr, "\n" ); }
-    show_error( e.s ); return 1;
+    show_error( e.msg ); return 1;
     }
 
   std::fprintf( stderr, "done\n" );
@@ -443,9 +444,9 @@ int RC::process_options()
   if( non_regular_found || read_error_found )
     {
     if( non_regular_found )
-      std::fprintf( stderr, "Warning: non-regular files will be ignored.\n" );
+      std::fprintf( stderr, "warning: Non-regular files will be ignored.\n" );
     if( read_error_found )
-      std::fprintf( stderr, "Warning: one or more files couldn't be read.\n" );
+      std::fprintf( stderr, "warning: One or more files couldn't be read.\n" );
     return 2;
     }
   return 0;
@@ -466,8 +467,8 @@ int RC::read_options( const int argc, const char * const argv[] )
 
 void RC::reset() throw()
   {
-  _editor_options.reset();
-  _default_buffer_options.reset();
+  editor_options_.reset();
+  default_buffer_options_.reset();
   priorities.reset();
   regex_options_vector.clear();
   }
