@@ -1,5 +1,6 @@
 /*  GNU Moe - My Own Editor
-    Copyright (C) 2005, 2006, 2007, 2008, 2009 Antonio Diaz Diaz.
+    Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011
+    Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,11 +49,13 @@ struct Lines_and_cursor
     : line( ln ), lines( lns ) {}
   };
 
-int _height, _width;
+char * line_buf_;
+int line_buf_size;
+int height_, width_;
 int clock_limit;		//   0: disable clock
 				//   1: showing help
 				// > 1: first line used by menu
-std::string _clock_string( " 0:00" );
+char clock_string_[6] = " 0:00";
 
 std::vector< Lines_and_cursor > lines_and_cursor_vector;
 
@@ -61,7 +64,7 @@ void dummy_endwin() throw() { endwin(); }
 
 void initialize_ncurses() throw()
   {
-  atexit( dummy_endwin );
+  std::atexit( dummy_endwin );
   initscr();			// initializes curses data structures
   raw();			// read a char at a time and disable signals
   keypad( stdscr, true );	// enables single value for function keys
@@ -80,7 +83,7 @@ void out_raw_string( const int line, const int col, const std::string & s,
                      const int len, int pos = 0, const chtype atr = A_NORMAL ) throw()
   {
   wmove( stdscr, line, col );
-  int x = col, limit = std::min( x + len, _width );
+  int x = col, limit = std::min( x + len, width_ );
   for( ; x < limit && pos < (int)s.size(); ++pos, ++x )
     {
     unsigned char ch = s[pos];
@@ -99,7 +102,7 @@ void out_string( const int line, const int col, const std::string & s,
                  const int len, int pos = 0 ) throw()
   {
   wmove( stdscr, line, col );
-  int x = col, limit = std::min( x + len, _width );
+  int x = col, limit = std::min( x + len, width_ );
   chtype atr = A_NORMAL;
   for( ; x < limit && pos < (int)s.size(); ++pos )
     {
@@ -159,7 +162,7 @@ void alt_key_loop( int key ) throw()
 void control_k_loop( int key )
   {
   Window & w = Window_vector::curwin();
-  const Buffer * old_bufferp = Block::bufferp();
+  const Buffer * const old_bufferp = Block::bufferp();
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
@@ -184,7 +187,7 @@ void control_k_loop( int key )
   }
 
 
-void control_o_loop( int key )
+void control_o_loop( int key, const char ** const msgp )
   {
   Window & w = Window_vector::curwin();
   w.show_status_line( ISO_8859::control_name( key ) );
@@ -202,6 +205,7 @@ void control_o_loop( int key )
     case 'C': Window_vector::center_line(); break;
     case 'D': Window_vector::remove_duplicate_lines(); break;
     case 'E': Window_vector::remove_duplicate_lines( true ); break;
+    case 'F': *msgp = Window_vector::search( 6, true, "", true ); break;
     case 'L': Window_vector::change_case( false ); break;
     case 'N': Window_vector::change_buffer_name(); break;
     case 'R': werase( stdscr ); wrefresh( stdscr ); repaint(); break;
@@ -211,7 +215,7 @@ void control_o_loop( int key )
   }
 
 
-int control_q_loop( int key )
+int control_q_loop( int key, const char ** const msgp )
   {
   Window & w = Window_vector::curwin();
   w.show_status_line( ISO_8859::control_name( key ) );
@@ -222,7 +226,7 @@ int control_q_loop( int key )
     {
     case KEY_F(2): Bufhandle_vector::save_all_named(); break;
     case 'C': if( Window_vector::close_and_exit( true ) == 0 ) return 0; break;
-    case 'U': Window_vector::copyright_update(); break;
+    case 'U': *msgp = Window_vector::copyright_update(); break;
     case 'X': if( Window_vector::close_and_exit( false ) == 0 ) return 0; break;
     }
   return -1;
@@ -247,28 +251,82 @@ void control_s_loop( int key, const std::string & version_msg ) throw()
   }
 
 
-void user_control_char()
+void user_data( const std::string & s, const int base )
   {
-  const std::string prompt( "Ctrl(A-Z) or 0-255 or 0x00-0xFF or 0-0377 or Escape sequence (^C to abort): " );
+  std::vector< int > buf;	// s is "big endian", buf is little endian
+  int last = s.size() - 1;
+  bool big_endian = false;
+  if( s[last] == '+' ) { --last; big_endian = true; }
+
+  if( base == 16 )
+    for( int i = last; i >= 2; i -= 2 )
+      {
+      const int lsd = ISO_8859::xtoi( s[i] );
+      const int msd = ( ( i > 2 ) ? ISO_8859::xtoi( s[i-1] ) : 0 );
+      if( lsd < 0 || msd < 0 )
+        { show_message( "Invalid hexadecimal data" ); return; }
+      buf.push_back( ( msd << 4 ) | lsd );
+      }
+  else
+    for( int i = 0; i <= last; ++i )
+      {
+      if( base == 10 && !std::isdigit( s[i] ) )
+        { show_message( "Invalid decimal data" ); return; }
+      if( base == 8 &&  !ISO_8859::isodigit( s[i] ) )
+        { show_message( "Invalid octal data" ); return; }
+      int carry = s[i] - '0';
+      for( unsigned int j = 0; j < buf.size(); ++j )
+        {
+        buf[j] = ( buf[j] * base ) + carry;
+        carry = buf[j] / 256;
+        buf[j] %= 256;
+        }
+      if( carry ) buf.push_back( carry );
+      }
+  if( !buf.size() ) buf.push_back( 0 );
+  if( big_endian )
+    for( unsigned int i = buf.size(); i > 0; --i )
+      Window_vector::add_char( buf[i-1], true );
+  else
+    for( unsigned int i = 0; i < buf.size(); ++i )
+      Window_vector::add_char( buf[i], true );
+  }
+
+
+void user_control_char( const int abort_key )
+  {
+  static std::vector< std::string > history;
+  const std::string prompt1( "(Ctrl)A-Z, Code(+) or Escape sequence (^C to abort): " );
+  const std::string prompt2( "           Code(+) or Escape sequence (^C to abort): " );
   int key;
+
   while( true )
     {
-    key = show_message( prompt, true, true );
-    if( key == KEY_F(1) ) Menu::help_menu( KEY_F(1) );
-    else break;
+    key = show_message( prompt1, false, true );
+    if( key == 8 || key == KEY_F(1) )
+      { wait_kbhit(); Menu::help_menu( 8, KEY_F(1) ); }
+    else if( key == 3 || key == abort_key ) { wait_kbhit(); return; }	// ^C
+    else if( key >= 256 || key == '\\' || std::isalnum( key ) ) break;
+    else wait_kbhit();
+    }
+  if( key < 256 && std::isalpha( key ) )
+    {
+    wait_kbhit();
+    key = ISO_8859::controlize( key ); if( key >= 32 ) key = -1;
     }
-  if( key < 0 || key >= 256 || key == 3 ) return;
-  if( key != '\\' && !std::isdigit( key ) )
-    { key = ISO_8859::controlize( key ); if( key >= 32 ) key = -1; }
   else
     {
-    std::vector< std::string > dummy;		// dummy history
-    dummy.push_back( std::string() );
-    dummy.back() += key;
-    if( get_string( prompt, dummy ) <= 0 ) return;
-    if( dummy.back()[0] == '\\' )
-      key = ISO_8859::escape( dummy.back(), 1 );
-    else if( !RC::parse_int( dummy.back(), key ) ) key = -1;
+    history.push_back( std::string() );
+    const int size = get_string( prompt2, history, abort_key );
+    if( size <= 0 ) return;
+    const std::string & s = history.back();
+    if( size >= 2 && s[0] == '\\' ) key = ISO_8859::escape( s, 1 );
+    else if( s[0] >= '1' && s[0] <= '9' )
+      { user_data( s, 10 ); return; }
+    else if( size >= 3 && s[0] == '0' && ( s[1] == 'x' || s[1] == 'X' ) )
+      { user_data( s, 16 ); return; }
+    else if( s[0] == '0' ) { user_data( s, 8 ); return; }
+    else key = -1;
     }
   if( key < 0 || key >= 256 ) show_message( "Invalid code" );
   else Window_vector::add_char( key, true );
@@ -276,31 +334,38 @@ void user_control_char()
 
 
 // `read_key' implements a buffer to speed up pastes, because `wgetch'
-// always calls `wrefresh'.
+// always calls `wrefresh'. `read_key' also converts '\r' to '\n'.
 //
 int read_key( const bool wait, const bool get_key )
   {
-  static std::vector< int > buf;
-  static unsigned int index = 0;	// index of key to be returned
+  enum { buf_size = 1024 };
+  static int buf[buf_size];
+  static int size = 0;			// number of keys in buf
+  static int index = 0;			// index of key to be returned
 
-  if( index >= buf.size() )		// buf is empty
+  if( index >= size )			// buf is empty
     {
-    buf.clear(); index = 0;
+    size = 0; index = 0;
     if( wait )
       {
       const int key = wgetch( stdscr );
-      if( key == ERR ) return -1; else buf.push_back( key );
+      if( key == ERR ) return -1; else buf[size++] = key;
       }
     nodelay( stdscr, true );
-    while( true )
+    while( size < buf_size )
       {
       const int key = wgetch( stdscr );
-      if( key == ERR ) break; else buf.push_back( key );
+      if( key == ERR ) break; else buf[size++] = key;
       }
     nodelay( stdscr, false );
     }
-  if( index < buf.size() )
-    { const int key = buf[index]; if( get_key ) ++index; return key; }
+  if( index < size )
+    {
+    const int key = buf[index]; if( get_key ) ++index;
+    if( key == '\r' ) return '\n';
+    if( key < 0 ) return -1;
+    return key;
+    }
   return -1;
   }
 
@@ -310,8 +375,10 @@ int read_key( const bool wait, const bool get_key )
 bool Screen::init()
   {
   initialize_ncurses();
-  getmaxyx( stdscr, _height, _width );
-  if( _height < 24 || _width < 80 ) return false;
+  getmaxyx( stdscr, height_, width_ );
+  if( height_ < 24 || width_ < 80 ) return false;
+  line_buf_size = width_ + 1;
+  line_buf_ = new char[line_buf_size];
   clock_limit = 0;
   clock_handler( 0 );
   Window_vector::init();
@@ -319,34 +386,46 @@ bool Screen::init()
   }
 
 
+int Screen::height() throw() { return height_; }
+
+int Screen::width() throw() { return width_; }
+
+int Screen::max_windows() throw() { return height_ / Window::min_height; }
+
+
+char * Screen::line_buf( int * const sizep ) throw()
+  { *sizep = line_buf_size; return line_buf_; }
+
+
 //void Screen::beep() throw() { ::beep(); wrefresh( stdscr ); }
 
 
-const std::string & Screen::clock_string() throw() { return _clock_string; }
+const char * Screen::clock_string() throw() { return clock_string_; }
 
 
 // updates the clock
 extern "C" void Screen::clock_handler( int sig ) throw()
   {
-  time_t dummy = time( 0 ); struct tm *t = localtime( &dummy );
+  const time_t tt = std::time( 0 );
+  const std::tm * const t = std::localtime( &tt );
   if( t )
     {
     int h = t->tm_hour % 12; if( h == 0 ) h = 12;
-    _clock_string[0] = ( h >= 10 ) ? '0' + ( h / 10 ) : ' ';
-    _clock_string[1] = '0' + ( h % 10 );
-    _clock_string[2] = (_clock_string[2] != ':') ? ':' : ' ';
-    _clock_string[3] = '0' + ( t->tm_min / 10 );
-    _clock_string[4] = '0' + ( t->tm_min % 10 );
+    clock_string_[0] = ( h >= 10 ) ? '0' + ( h / 10 ) : ' ';
+    clock_string_[1] = '0' + ( h % 10 );
+    clock_string_[2] = (clock_string_[2] != ':') ? ':' : ' ';
+    clock_string_[3] = '0' + ( t->tm_min / 10 );
+    clock_string_[4] = '0' + ( t->tm_min % 10 );
     if( sig )
       {
       Point old_cursor; save_cursor( old_cursor );
       if( clock_limit == 1 )
-        out_raw_string( 0, 60, _clock_string, 5, 0, A_REVERSE );
+        out_raw_string( 0, 60, clock_string_, 5, 0, A_REVERSE );
       else for( int i = 0; i < Window_vector::windows(); ++i )
         {
         const Point cursor = Window_vector::window( i ).clock_position();
         if( cursor.line >= clock_limit ) break;
-        out_raw_string( cursor.line, cursor.col, _clock_string, 5, 0, A_REVERSE );
+        out_raw_string( cursor.line, cursor.col, clock_string_, 5, 0, A_REVERSE );
         }
       wmove( stdscr, old_cursor.line, old_cursor.col );
       wrefresh( stdscr ); alarm( 1 );
@@ -371,14 +450,13 @@ int Screen::wait_kbhit( const int new_clock_limit, const bool get_key,
     if( new_clock_limit < 0 )
       move_to( Window_vector::curwin().absolute_cursor() );
 
-    clock_limit = ( new_clock_limit >= 0 ) ? new_clock_limit : _height;
+    clock_limit = ( new_clock_limit >= 0 ) ? new_clock_limit : height_;
     if( clock_limit > 0 ) alarm( 1 );
     wrefresh( stdscr );
     }
 
-  int key = read_key( true, get_key );
+  const int key = read_key( true, get_key );
   alarm( 0 );
-  if( key < 0 ) key = -1;
   return key;
   }
 
@@ -389,10 +467,9 @@ int Screen::wait_kbhit( const int new_clock_limit, const bool get_key,
 //
 int Screen::get_string( const std::string & prompt,
                         std::vector< std::string > & history,
-                        const int abort_key,
-                        const bool allow_copy, const bool usetab )
+                        const int abort_key, const bool usetab )
   {
-  const int line = _height - 1, lines = 1;
+  const int line = height_ - 1, lines = 1;
   save_lines_and_cursor( line, lines );
 
   const std::string original( history.back() );
@@ -404,8 +481,8 @@ int Screen::get_string( const std::string & prompt,
   bool modified = false;
   while( key >= 0 )
     {
-    int pos = scol; pos -= ( _width - 1 ); if( pos < 0 ) pos = 0;
-    out_raw_string( line, 0, s, _width, pos );
+    int pos = scol; pos -= ( width_ - 1 ); if( pos < 0 ) pos = 0;
+    out_raw_string( line, 0, s, width_, pos );
     wmove( stdscr, line, scol - pos );
     do key = wait_kbhit( line ); while( key < 0 );
     if( key == abort_key ) key = 3;
@@ -425,7 +502,7 @@ int Screen::get_string( const std::string & prompt,
                    s.insert( prompt.size(), name );
                    }
                  break;
-      case '\r': {
+      case '\n': {
                  std::string name( s, prompt.size(), s.size() );
                  if( !usetab || !name.size() )
                    { key = -2; history.back() = name; }
@@ -434,13 +511,9 @@ int Screen::get_string( const std::string & prompt,
                      else { key = -2; history.back() = name; } }
                  }
                  break;
-      case  25 : if( s.size() > prompt.size() )			// ^Y
-                   { s.resize( prompt.size() ); scol = s.size();
-                     modified = true; }
-                 break;
-      case   8 :						// ^H
+      case 8        :						// ^H
       case KEY_F(1) : Menu::help_menu( 8, KEY_F(1) ); break;
-      case KEY_F(9) : if( allow_copy && Block::valid() )
+      case KEY_F(9) : if( Block::valid() )
                         {
                         std::string tmp;
                         Block::bufferp()->to_string( Block::begin(), Block::end(), tmp );
@@ -449,14 +522,18 @@ int Screen::get_string( const std::string & prompt,
                           { s.insert( scol, tmp ); scol += tmp.size(); modified = true; }
                         }
                       break;
-      case KEY_DC   : if( scol < s.size() )
-                        { s.erase( scol, 1 ); modified = true; } break;
+      case 127          :					// delete
       case KEY_BACKSPACE: if( scol > prompt.size() )
                             { s.erase( --scol, 1 ); modified = true; } break;
+      case KEY_DC   : if( scol < s.size() )
+                        { s.erase( scol, 1 ); modified = true; } break;
       case KEY_HOME : scol = prompt.size(); break;
       case KEY_END  : scol = s.size(); break;
       case KEY_LEFT : if( scol > prompt.size() ) { --scol; } break;
       case KEY_RIGHT: if( scol < s.size() ) { ++scol; } break;
+      case 25       : if( s.size() > prompt.size() )		// ^Y
+                        { s.resize( prompt.size() ); scol = s.size();
+                          modified = true; }		// fall through
       case KEY_PPAGE:
       case KEY_UP   : if( idx > 0 )
         {
@@ -468,7 +545,7 @@ int Screen::get_string( const std::string & prompt,
           else if( idx == history.size() - 1 ) history[idx].clear();
           else history.erase( history.begin() + idx );
           }
-        if( key == KEY_UP ) --idx; else idx = 0;
+        if( key == KEY_PPAGE ) idx = 0; else --idx;
         s.erase( prompt.size(), s.size() ); s += history[idx]; scol = s.size();
         } break;
       case KEY_NPAGE:
@@ -500,13 +577,6 @@ int Screen::get_string( const std::string & prompt,
   }
 
 
-int Screen::height() throw() { return _height; }
-
-int Screen::width() throw() { return _width; }
-
-int Screen::max_windows() throw() { return _height / Window::min_height; }
-
-
 void Screen::move_to( const Point & cursor ) throw()
   { wmove( stdscr, cursor.line, cursor.col ); }
 
@@ -522,7 +592,7 @@ void Screen::save_lines_and_cursor( const int line, const int lines )
 
   save_cursor( lc.cursor );
   for( int y = line; y < line + lines; ++y )
-    for( int x = 0; x < _width; ++x )
+    for( int x = 0; x < width_; ++x )
       lc.buf.push_back( mvinch( y, x ) );
   }
 
@@ -532,7 +602,7 @@ void Screen::restore_lines_and_cursor() throw()
   const Lines_and_cursor & lc = lines_and_cursor_vector.back();
 
   for( int y = lc.line, i = 0; y < lc.line + lc.lines; ++y )
-    for( int x = 0; x < _width; ++x )
+    for( int x = 0; x < width_; ++x )
       { wmove( stdscr, y, x ); waddch( stdscr, lc.buf[i++] ); }
   wmove( stdscr, lc.cursor.line, lc.cursor.col );
   lines_and_cursor_vector.pop_back();
@@ -543,20 +613,20 @@ void Screen::out_buffer_line( const Buffer & buffer, Point c, const int line ) t
   {
   wmove( stdscr, line, 0 );
   if( c.line < 0 || c.line >= buffer.lines() )
-    { rep_addch( _width, ' ' ); return; }
+    { rep_addch( width_, ' ' ); return; }
   Point pc, p = buffer.to_pointer( c, pc );
   if( pc.col < c.col ) pc.col = c.col;		// landed in a tab
 
-  for( int i = 0; i < _width; ++i, ++c.col, ++p.col )
+  for( int i = 0; i < width_; ++i, ++c.col, ++p.col )
     {
     int ch = buffer[p];
     if( ch < 0 || ( p == buffer.eol( p ) && ch == '\n' ) )
-      { rep_addch( _width - i, ' ' ); break; }
+      { rep_addch( width_ - i, ' ' ); break; }
     int block_attr = Block::in_block( buffer, p ) ? A_REVERSE : 0;
     if( ch == '\t' )
       {
       ch = ' ';
-      while( i + 1 < _width && c.col % 8 != 7 )
+      while( i + 1 < width_ && c.col % 8 != 7 )
         { waddch( stdscr, ch | block_attr ); ++i; ++c.col; }
       }
     else if( ch < 32 ) ch = ( ch + '@' ) | A_UNDERLINE;
@@ -570,17 +640,17 @@ void Screen::out_buffer_line( const Buffer & buffer, Point c, const int line ) t
 bool Screen::out_line( const std::string & s, const int line, const bool shift,
                        const char mode ) throw()
   {
-  if( line < 0 || line >= _height ) return false;
-  int pos = s.size(); pos -= ( _width - 1 );
+  if( line < 0 || line >= height_ ) return false;
+  int pos = s.size(); pos -= ( width_ - 1 );
   if( !shift || pos < 0 ) pos = 0;
-  if( !mode ) out_string( line, 0, s, _width, pos );
+  if( !mode ) out_string( line, 0, s, width_, pos );
   else
     {
     chtype atr = A_NORMAL;
     if( mode == 'b' ) atr = A_BOLD;
     else if( mode == 'i' ) atr = A_REVERSE;
     else if( mode == 'u' ) atr = A_UNDERLINE;
-    out_raw_string( line, 0, s, _width, pos, atr );
+    out_raw_string( line, 0, s, width_, pos, atr );
     }
   return true;
   }
@@ -589,22 +659,22 @@ bool Screen::out_line( const std::string & s, const int line, const bool shift,
 bool Screen::out_menu_line( const std::string & s, const int line,
                             const int hbegin, const int hend ) throw()
   {
-  if( line < 0 || line >= _height ) return false;
+  if( line < 0 || line >= height_ ) return false;
   if( hbegin >= 0 && hbegin < hend && hend <= (int)s.size() )
     {
     if( hbegin > 0 )
       out_raw_string( line, 0, s, hbegin, 0, A_NORMAL );
-    if( hbegin < _width )
+    if( hbegin < width_ )
       out_raw_string( line, hbegin, s, hend - hbegin, hbegin, A_REVERSE );
-    if( hend < _width )
-      out_raw_string( line, hend, s, _width, hend, A_NORMAL );
+    if( hend < width_ )
+      out_raw_string( line, hend, s, width_, hend, A_NORMAL );
     }
-  else out_raw_string( line, 0, s, _width, 0, A_NORMAL );
+  else out_raw_string( line, 0, s, width_, 0, A_NORMAL );
   return true;
   }
 
 
-void Screen::repaint( const Buffer * bufp ) throw()
+void Screen::repaint( const Buffer * const bufp ) throw()
   {
   for( int i = 0; i < Window_vector::windows(); ++i )
     {
@@ -621,12 +691,12 @@ void Screen::repaint( const Buffer * bufp ) throw()
 int Screen::show_message( const std::string & s, const bool get_key,
                           const bool take_cursor )
   {
-  const int line = _height - 1;
+  const int line = height_ - 1;
 
   save_lines_and_cursor( line, 1 );
 
-  out_string( line, 0, s, _width );
-  wmove( stdscr, line, std::min( (int)s.size(), _width - 1 ) );
+  out_string( line, 0, s, width_ );
+  wmove( stdscr, line, std::min( (int)s.size(), width_ - 1 ) );
   int key;
   do key = wait_kbhit( take_cursor ? line : -1, get_key ); while( key < 0 );
 
@@ -635,7 +705,7 @@ int Screen::show_message( const std::string & s, const bool get_key,
   }
 
 
-void Screen::show_status_lines( const Buffer * bufp ) throw()
+void Screen::show_status_lines( const Buffer * const bufp ) throw()
   {
   for( int i = 0; i < Window_vector::windows(); ++i )
     if( !bufp || bufp == &Window_vector::window( i ).buffer() )
@@ -669,21 +739,23 @@ int Screen::user_loop( const std::string & version_msg ) throw()
     {
     const int key = wait_kbhit( -1, true, force_refresh );
     if( key < 0 ) continue;
+    const char * msg = 0;
     force_refresh = false;
     { Window & w = Window_vector::curwin();
     switch( key )
       {
-      case  0: Block::toggle_marking( w.buffer(), w.pointer() ); break;	// ^SPACE
+      case  0: msg = Block::toggle_marking( w.buffer(), w.pointer() );
+               break;						// ^SPACE
       case  2: Window_vector::reformat(); break;		// ^B
       case  3: if( !Window_vector::close( true ) ) retval = 0; break;	// ^C
-      case  6: Window_vector::search( key ); break;		// ^F
-      case  7: Window_vector::search( key, true ); break;	// ^G
+      case  6: msg = Window_vector::search( key ); break;	// ^F
+      case  7: msg = Window_vector::search( key, true ); break;	// ^G
       case 11: control_k_loop( key ); break;			// ^K
-      case 15: control_o_loop( key ); break;			// ^O
-      case 16: user_control_char(); break;			// ^P
-      case 17: retval = control_q_loop( key ); break;		// ^Q
+      case 15: control_o_loop( key, &msg ); break;		// ^O
+      case 16: user_control_char( key ); break;			// ^P
+      case 17: retval = control_q_loop( key, &msg ); break;	// ^Q
       case 19: control_s_loop( key, version_msg ); break;	// ^S
-      case 23: Window_vector::search_word( key ); break;	// ^W
+      case 23: msg = Window_vector::search_word( key ); break;	// ^W
       case 24: if( !Window_vector::close( false ) ) retval = 0; break;	// ^X
       case 25: Window_vector::delete_line(); break;		// ^Y
       case 27: alt_key_loop( key ); break;			// Alt or Esc
@@ -700,6 +772,7 @@ int Screen::user_loop( const std::string & version_msg ) throw()
       case KEY_F(10): Menu::options_menu( w.buffer(), key ); break;
       case KEY_F(11): Window_vector::bufhandle_menu( key ); break;
       case KEY_F(12): Window_vector::last_visited(); break;
+      case 127:							// delete
       case KEY_BACKSPACE: Window_vector::delete_char( true ); break;
       case KEY_DC   : Window_vector::delete_char(); break;
       case KEY_IC   : w.buffer().options.overwrite = !w.buffer().options.overwrite;
@@ -712,16 +785,16 @@ int Screen::user_loop( const std::string & version_msg ) throw()
       case KEY_DOWN : w.move_vertical( +1 ); break;
       case KEY_LEFT : w.goto_pprev(); break;
       case KEY_RIGHT: w.goto_pnext(); break;
-      default       : if( key < 256 )
-                        { if( key == '\r' || key == '\n' ) force_refresh = true;
-                          Window_vector::add_char( key ); }
+      default       : if( key == '\n' ) force_refresh = true;
+                      Window_vector::add_char( key );
       } }
     if( retval >= 0 ) break;
     { Window & w = Window_vector::curwin();
     if( Block::follow_marking( w.buffer(), w.pointer() ) )
       repaint( &w.buffer() ); }
+    if( msg ) show_message( msg );
     }
-  wmove( stdscr, _height - 1, 0 );	// clear bottom line on exit
+  wmove( stdscr, height_ - 1, 0 );	// clear bottom line on exit
   wclrtoeol( stdscr ); wrefresh( stdscr );
   return retval;
   }
